/*
   FILE:							       	
	rtl_tasks.c							       
								       	
   DESCRIPTION:							       	
	RT-Linux side of a sample application using the POSIX trace
	support.

	See README file for details.
					       				
   Copyright (C) 2001
	Ana Garcia Fornes
	Andres Terrasa Barrena
	Ignacio Paches Giner
	Agustin Espinosa Minguet

  This is free software;  you can  redistribute it  and/or modify it under
  terms of the  GNU General Public License as published  by the Free Soft-
  ware  Foundation;  either version 2,  or (at your option) any later ver-
  sion.  It is distributed in the hope that it will be useful, but WITH-
  OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY
  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for  more details.  You should have  received  a copy of the GNU General
  Public License;  see file COPYING.  If not, write to  the Free Software 
  Foundation,  59 Temple Place - Suite 330,  Boston, MA 02111-1307, USA.

  LAST REVISION:
        September 2001

*/

#include <rtl.h>
#include <time.h>
#include <pthread.h>
#include <rtl_sched.h>
#include <trace.h>

#define _DEBUG

static trace_id_t       trid;
static pthread_t        writer_task, reader_task;


void *writer(void *task_period) {
  trace_event_id_t ev_char, ev_int, ev_string;
  int   i;
  int   j;
  char  s[32] ="Hello world!";
  char  c;
  int   k;
  long      period;
  struct timespec next_period;


  period = (long) task_period;
  clock_gettime(CLOCK_MONOTONIC, &next_period);

  // Create new event types:
  posix_trace_eventid_open ("user event string", &ev_string); 
  posix_trace_eventid_open ("user event char",   &ev_char);
  posix_trace_eventid_open ("user event int",    &ev_int);

  c = 'A';
  k = 0;

  for (i=0; i<20; i++) {

    timespec_add_ns(&next_period, period);
    clock_nanosleep(CLOCK_MONOTONIC, TIMER_ABSTIME, &next_period, NULL);

    for (j=0; j<700000; j++);
   
    posix_trace_event(ev_char, (void *) &c, sizeof(char));
    posix_trace_event(ev_int, (void *) &k, sizeof(int));
    posix_trace_event(ev_string, s, sizeof(s));

    // Values for next loop:
    c += 1;
    k += 1;
    
  }
  return (void *) 0;
}






void *reader(void *dummy) {
  int             error;
  trace_attr_t    trace_attr;
  char            str[TRACE_NAME_MAX];
  struct posix_trace_event_info event;
  char            data[64];
  size_t          datalen;
  int             unavailable;
  int             *ent;
  char            *car;
  trace_event_id_t evid;

  struct timespec next_period;


  rtl_printf(" (reader) Getting trace attributes:\n");
  error = posix_trace_get_attr(trid, &trace_attr);
  
  error = posix_trace_attr_getgenversion(&trace_attr, str);
  rtl_printf( " (reader) Trace generation version (%d): %s\n", error, str);

  rtl_printf(" (reader) Retrieving event types:\n");
  posix_trace_eventtypelist_rewind(trid);
  posix_trace_eventtypelist_getnext_id (trid, &evid, &unavailable);
  clock_gettime(CLOCK_MONOTONIC, &next_period);
  while (! unavailable) {
    timespec_add_ns(&next_period, (long) 100000);
    clock_nanosleep(CLOCK_MONOTONIC, TIMER_ABSTIME, &next_period, NULL);
    posix_trace_eventid_get_name (trid, evid, str);
    rtl_printf(" (reader)     - event %d name %s\n", evid, str);
    posix_trace_eventtypelist_getnext_id (trid, &evid, &unavailable);
  }

  rtl_printf("\n (reader) Now retrieving events:\n\n");
    
  clock_gettime(CLOCK_MONOTONIC, &next_period);
  next_period.tv_sec += 2;
  clock_nanosleep(CLOCK_MONOTONIC, TIMER_ABSTIME, &next_period, NULL);

  error = 0; unavailable = 0;

  while (! error && ! unavailable) {

    error = posix_trace_getnext_event(trid, 
				      &event, 
				      &data, 
				      sizeof(data), 
				      &datalen, 
				      &unavailable);
    
    if(error) {
      rtl_printf(" (reader) No more events (%d). Exiting\n", error);
      break;

    } else if (unavailable) {
      rtl_printf( " (reader) Event unavailable\n");

    } else {
      posix_trace_eventid_get_name (trid, event.posix_event_id, str);

      // Now switch depending on the event type (name):
      if (!strcmp(str,"user event char")) {
	car = (char *) data;
	rtl_printf(" (reader) [%ld.%ld]. Event %d (%s). Data=%c\n", 
		    event.posix_timestamp.tv_sec, 
		    event.posix_timestamp.tv_nsec,
		    event.posix_event_id, str, *car);
      } 
      else if (!strcmp(str,"user event int")) {
	ent = (int *) data;
	rtl_printf(" (reader) [%ld.%ld]. Event %d (%s). Data=%d\n", 
		    event.posix_timestamp.tv_sec, 
		    event.posix_timestamp.tv_nsec,
		    event.posix_event_id, str, *ent);
      } 
      else if (!strcmp(str,"user event string")) {
	rtl_printf(" (reader) [%ld.%ld]. Event %d (%s). Data=%s\n", 
		    event.posix_timestamp.tv_sec, 
		    event.posix_timestamp.tv_nsec,
		    event.posix_event_id, str, (char *) data);
      } 
      else {
	rtl_printf(" (reader) [%ld.%ld]. Event %d (%s). Data=#UNKNOWN#\n", 
		    event.posix_timestamp.tv_sec, 
		    event.posix_timestamp.tv_nsec,
		    event.posix_event_id, str);
      } 
    }
  }
  rtl_printf(" (reader) Exiting. error=%d, unavailable = %d \n", 
		  error, unavailable);

  return (void *) 0;
}




 


int init_module(void) {

  trace_event_set_t set;
  trace_attr_t      attr;
  int               error;


  // Create and set the trace attribute:
  error = posix_trace_attr_init(&attr);
  error = posix_trace_attr_setstreamfullpolicy (&attr, POSIX_TRACE_UNTIL_FULL);
  error = posix_trace_attr_setmaxdatasize(&attr, 64);
  error = posix_trace_attr_setstreamsize(&attr, 40960);
  
 // Create the stream:
  error = posix_trace_create(0, &attr, &trid); 
  if (error) {
    return -1;
  }

  // Set the stream filter to only record user events:
  posix_trace_eventset_fill(&set, POSIX_TRACE_SYSTEM_EVENTS);
  error = posix_trace_set_filter(trid, (const trace_event_set_t *) &set, 
  			 POSIX_TRACE_SET_EVENTSET);

  // Start tracing:
  error = posix_trace_start(trid);
  if (error) {
    return -1;
  }


  // Create the 'writer' task (the one which traces user events):
  pthread_create (&writer_task, NULL, writer, (void *) 1000000000); 

  // Create the 'reader' task (awakes only once):
  pthread_create (&reader_task, NULL, reader, 0); 

  return 0;
}



void cleanup_module(void) {

#ifdef _DEBUG
  rtl_printf("rtl_tasks: CLEANUP!!!\n");
#endif

  // Stop and shutdown the stream:
  posix_trace_shutdown(trid); 

  // Delete the tasks:
  pthread_cancel(writer_task);
  pthread_cancel(reader_task);

  pthread_join(writer_task,NULL); 
  pthread_join(reader_task,NULL); 


}
