#include <pthread.h>
#include <sys/types.h>
#include <trace.h>

#ifndef __METRICS_H
#define __METRICS_H


///////////////////////////////////////////////
//  INTERNALS
///////////////////////////////////////////////


#include <metrics/metrics_internals.h>


///////////////////////////////////////////////
//  INTERFACE
///////////////////////////////////////////////


typedef int metrics_metric_id_t;

typedef struct {
  int s [METRICS_MAX];
  int current_pos;
} metrics_metricset_t;

typedef struct {
  metrics_metric_id_t     metric_id;
  int             thread_id;
  int             object_id;
  struct timespec duration;
  struct timespec begin;
  struct timespec end;
  int             events_count;
  int             id;

} metrics_measurement_t;



int
metrics_metric_open (const char          *metric_name,
		     metrics_metric_id_t *metric_id);

int
metrics_metric_create (const char          *metric_name,
		       const char          *metric_description,
		       const char          *initial_event,
		       const char          *final_event,
		       metrics_metric_id_t *metric_id);

int
metrics_metric_getname (metrics_metric_id_t  metric_id,
			char                *metric_name);

int
metrics_metric_getdescription (metrics_metric_id_t  metric_id,
			       char                *metric_description);


int
metrics_metricset_empty (metrics_metricset_t *set);

int
metrics_metricset_fill (metrics_metricset_t *set);

int
metrics_metricset_add (metrics_metric_id_t  metric_id,
		       metrics_metricset_t *set);

int
metrics_metricset_del (metrics_metric_id_t  metric_id,
		       metrics_metricset_t *set);

int
metrics_metricset_ismember (metrics_metric_id_t        metric_id,
			    const metrics_metricset_t *set,
			    int                       *ismember);

int 
metrics_metricset_iterator_getnext (metrics_metricset_t *set,
				    metrics_metric_id_t *metric_id,
				    int                 *unavailable);

int
metrics_metricset_iterator_rewind (metrics_metricset_t *set);


int
metrics_init (trace_id_t                 trid,
	      const metrics_metricset_t *set);

int
metrics_destroy (void);

int
metrics_getnext_measurement (metrics_measurement_t *result,
			     int                   *unavailable);


int
metrics_trygetnext_measurement (metrics_measurement_t *result,
				int                   *unavailable);


int
metrics_timedgetnext_measurement (metrics_measurement_t *result,
				  int                   *unavailable,
				  const struct timespec *abs_timeout);




#endif // __METRICS_H
