#define TRUE  1
#define FALSE 0


#define METRICS_MAX              32
#define METRICS_NAME_MAX         64
#define METRICS_DESCRIPTION_MAX 255

#define MTRI_NO_METRIC        -1

#define MTRI_NULL_THREAD (int) -1
#define MTRI_NULL_OBJECT (int) -1

#define MTRI_INDEX_MAX        128
#define MTRI_TRACE_EVENTS_MAX 128

typedef long long mtri_time_t;

// #define DO_MTRI_DEBUG



#define NSECS_PER_SEC 1000000000

#define timespec_normalize(t) {\
	if ((t)->tv_nsec >= NSECS_PER_SEC) { \
		(t)->tv_nsec -= NSECS_PER_SEC; \
		(t)->tv_sec++; \
	} else if ((t)->tv_nsec < 0) { \
		(t)->tv_nsec += NSECS_PER_SEC; \
		(t)->tv_sec--; \
	} \
}

#define timespec_add(t1, t2) do { \
	(t1)->tv_nsec += (t2)->tv_nsec;  \
	(t1)->tv_sec += (t2)->tv_sec; \
	timespec_normalize(t1);\
} while (0)

#define timespec_sub(t1, t2) do { \
	(t1)->tv_nsec -= (t2)->tv_nsec;  \
	(t1)->tv_sec -= (t2)->tv_sec; \
	timespec_normalize(t1);\
} while (0)


#define STATES_MAX   10
#define NO_STATUS    -1


typedef int mtri_event_id_t;

typedef enum {
  EMPTY_TRANSITION, LAMBDA, 
NORMAL

} mtri_transition_kind_t;




typedef enum {
  EMPTY_STATE, IN, OUT, CANCEL, END

} mtri_state_kind_t;


typedef enum {
  ANY_THREAD, SELF_THREAD, OTHER_THREAD

} thread_spec_t;


typedef enum {
  ANY_OBJECT, SELF_OBJECT, OTHER_OBJECT

} object_spec_t;


typedef enum {
  NO_KIND, SYSTEM_WIDE, THREAD_RELATED, MUTEX_RELATED, COND_RELATED

} mtri_automaton_kind_t;


typedef struct {
  mtri_state_kind_t kind;
} mtri_state_descriptor_t;


typedef struct {
  int                    to_state;
  mtri_transition_kind_t kind;
  mtri_event_id_t        event_id;
  thread_spec_t          thread;
  object_spec_t          object;
  void                  *next;

} mtri_transition_descriptor_t;



typedef struct {
  int                          metric_id;
  mtri_automaton_kind_t        kind;
  int                          num_states;
  mtri_state_descriptor_t      state [STATES_MAX];
  mtri_transition_descriptor_t *transition [STATES_MAX];
} mtri_automaton_class_descriptor_t;



typedef struct {
  mtri_automaton_class_descriptor_t *class;
  int                                thread;
  int                                object;
  int                                measuring;
  int                                in_segment;
  int                                current_state;
  struct timespec                    begin;
  struct timespec                    end;  
  struct timespec                    partial_first;
  struct timespec                    partial_last;
  struct timespec                    sum;
  int                                events_count;
  void                              *next;

} mtri_automaton_instance_descriptor_t;




typedef struct {
  mtri_event_id_t  event_id;
  struct timespec  timestamp;
  int              thread;
  int              object;

} mtri_input_tuple_t;


int 
mtri_automata_initialize (void);

int
mtri_create_automaton (mtri_automaton_class_descriptor_t *class,
		       int                                thread,
		       int                                object);

int
mtri_process_one_event (mtri_automaton_instance_descriptor_t *aut,
			mtri_input_tuple_t                   *tuple,
			int                                  *measurement_done);





typedef char mtri_metric_name_t        [METRICS_NAME_MAX];

typedef char mtri_metric_description_t [METRICS_DESCRIPTION_MAX];

typedef struct  {
  mtri_metric_name_t                name;
  mtri_metric_description_t         description;
  mtri_automaton_class_descriptor_t automaton;

} mtri_metric_descriptor_t;



