/*
   FILE:							       	
	trace/comm.h							       
								       	
   DESCRIPTION:							       	
	Private header for communication between the RT-Linux and Linux
	sides of the POSIX trace support.
					       				
   Copyright (C) 2001
	Ana Garcia Fornes
	Andres Terrasa Barrena
	Ignacio Paches Giner
	Agustin Espinosa Minguet

  This is free software;  you can  redistribute it  and/or modify it under
  terms of the  GNU General Public License as published  by the Free Soft-
  ware  Foundation;  either version 2,  or (at your option) any later ver-
  sion.  It is distributed in the hope that it will be useful, but WITH-
  OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY
  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for  more details.  You should have  received  a copy of the GNU General
  Public License;  see file COPYING.  If not, write to  the Free Software 
  Foundation,  59 Temple Place - Suite 330,  Boston, MA 02111-1307, USA.

  LAST REVISION:
        April 2002

*/

#ifndef __TRACE_POSIX_TRACE_COMM_H
#define __TRACE_POSIX_TRACE_COMM_H


// First include all the POSIX definitions (public and private)
#include "../trace.h"




/*
  RT-FIFO DEFINITIONS.
  
  The trace support at the LINUX level is implemented by using
  RT-FIFOS
*/ 


#define TRACE_FIFO_MAGIC 0x4ffa



typedef enum fifo_trace_op_t {
  FIFO_TRACE_NO_OPERATION,   
  FIFO_TRACE_EVENTID_OPEN,
  FIFO_TRACE_EVENT
} fifo_trace_op_t;



typedef struct fifo_request_header_t {
  int               magic;
  fifo_trace_op_t   operation;

  union rdata {

    char              eventname[TRACE_EVENT_NAME_MAX];

    struct {
      trace_event_id_t eventid;
      size_t           data_len;
      int              calling_address;
      char             data[MAX_FIFO_DATA_SIZE];
    } tracevent;
  } rdata;

} fifo_request_t;



typedef struct fifo_answer_struct {
  int           magic;
  int           return_value;
  union data {

    trace_event_id_t       eventid;

  } data;
} fifo_answer_t;


/*
   Communication from RT-Linux to a Linux process to flush events into
   a device (normally a FIFO, though the implementation does not
   impose this). Somebody is supposed to be listening at the "other
   side" of the device (in the host machine?) in order to read the
   flushing info and translate it into a valid log file.
*/

#define FLUSH_LOG_STARTING  0xabc1
#define FLUSH_LOG_DONE      0xabc2  
#define FLUSH_LOG_HEADER    0xabc3
#define FLUSH_LOG_EVENT     0xabc4


typedef struct flush_header_struct {
  int   type;      // Type of message header 
  int   add_bytes; // Additional bytes to be read after the header
} flush_header_t;






#endif
