/*
   FILE:
						       	
	test_ptrace_api1_module.c
								       	
   DESCRIPTION:			
				       	
        Test 1 for POSIX Tracing API in RT-Linux.

	This test revises the functions which manipulate (get/set)
	all the trace stream attributes.

  				
   Copyright (C) 2004
	Ana Garcia Fornes
	Andres Terrasa Barrena
	Ignacio Paches Giner
	Agustin Espinosa Minguet

  This is free software;  you can  redistribute it  and/or modify it under
  terms of the  GNU General Public License as published  by the Free Soft-
  ware  Foundation;  either version 2,  or (at your option) any later ver-
  sion.  It is distributed in the hope that it will be useful, but WITH-
  OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY
  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for  more details.  You should have  received  a copy of the GNU General
  Public License;  see file COPYING.  If not, write to  the Free Software 
  Foundation,  59 Temple Place - Suite 330,  Boston, MA 02111-1307, USA.

  LAST REVISION:
        September 2004

*/

#include <rtl.h>
#include <time.h>
#include <trace.h>
#include <asm/errno.h>


#define TEST_NAME "ptrace_test_api1"


MODULE_LICENSE("GPL");
MODULE_AUTHOR("Andres Terrasa");
MODULE_DESCRIPTION("API Test 1 for POSIX Tracing");



int init_module(void) {

  trace_attr_t   attr;
  int            error;
  char           str[TRACE_NAME_MAX];
  struct timespec time;
  struct timespec resolution;
#ifdef _POSIX_TRACE_INHERITANCE
  int            inher_policy;
#endif
  int            full_policy;
  size_t         size;
  size_t         size_set;

  
  // Create the trace attribute:
  if ((error = posix_trace_attr_init(&attr)) != 0) {
    rtl_printf("%s: [BROK] :posix_trace_attr_init returned %d\n", 
	       TEST_NAME, error);
    return -1;
  }
  
  // Set/Get the trace attribute:
  if ((error = posix_trace_attr_getgenversion(&attr, str)) != 0) {
    rtl_printf("%s: [BROK] :posix_trace_attr_getgenversion returned %d\n", 
	       TEST_NAME, error);
    return -1;
  }
  
  if ((error = posix_trace_attr_setname(&attr, TEST_NAME)) != 0) {
    rtl_printf("%s: [BROK] :posix_trace_attr_setname returned %d\n", 
	       TEST_NAME, error);
    return -1;
  }
      
  if ((error = posix_trace_attr_getname(&attr, str)) != 0) {
    rtl_printf("%s: [BROK] :posix_trace_attr_getname returned %d\n", 
	       TEST_NAME, error);
    return -1;
    if (strcmp(TEST_NAME, str)) {
      rtl_printf("%s: [BROK] :attribute name set and get different\n");
      return -1;
    }
  }
  
  if ((error = posix_trace_attr_getcreatetime(&attr, &time)) != 0) {
    rtl_printf("%s: [BROK] :posix_trace_attr_getcreatetime returned %d\n", 
	       TEST_NAME, error);
    return -1;
  }
  
  
  if ((error = posix_trace_attr_getclockres(&attr, &resolution)) != 0) {
    rtl_printf("%s: [BROK] :posix_trace_attr_getclockres returned %d\n", 
	       TEST_NAME, error);
    return -1;
  }
  
#ifdef _POSIX_TRACE_INHERITANCE
  if ((error = posix_trace_attr_getinherited(&attr, &inher_policy)) != 0) {
    rtl_printf("%s: [BROK] :posix_trace_attr_getinheritance returned %d\n", 
	       TEST_NAME, error);
    return -1;
    if (inher_policy != POSIX_TRACE_INHERITED &&
	inher_policy != POSIX_TRACE_CLOSE_FOR_CHILD) {
      rtl_printf("%s: [BROK] :wrong policy returned by posix_trace_attr_getinherited\n", 
		 TEST_NAME);
      return -1;
    }
  }
  

  if ((error = posix_trace_attr_setinherited(&attr, POSIX_TRACE_INHERITED)) != 0) {
    rtl_printf("%s: [BROK] :posix_trace_attr_setinheritance returned %d\n", 
	       TEST_NAME, error);
    return -1;
  }

  posix_trace_attr_getinherited(&attr, &inher_policy);
  if (inher_policy != POSIX_TRACE_INHERITED) {
    rtl_printf("%s: [BROK] :inheritance attribute set and get different\n");
    return -1;
  }

#endif


  if ((error = posix_trace_attr_getstreamfullpolicy(&attr, &full_policy)) != 0) {
    rtl_printf("%s: [BROK] :posix_trace_attr_getstreamfullpolicy returned %d\n", 
	       TEST_NAME, error);
    return -1;
    if (full_policy != POSIX_TRACE_LOOP 
	&& full_policy != POSIX_TRACE_UNTIL_FULL
#ifdef _POSIX_TRACE_TOLOG
	&& full_policy != POSIX_TRACE_FLUSH
#endif
	) {
      rtl_printf("%s: [BROK] :wrong policy returned by posix_trace_attr_getstreamfullpolicy\n", 
	       TEST_NAME);
      return -1;
    }
  }


  if ((error = posix_trace_attr_setstreamfullpolicy(&attr, POSIX_TRACE_UNTIL_FULL)) != 0) {
    rtl_printf("%s: [BROK] :posix_trace_attr_setstreamfullpolicy returned %d\n", 
	       TEST_NAME, error);
    return -1;
  }  
   
  posix_trace_attr_getstreamfullpolicy(&attr, &full_policy);
  if (full_policy != POSIX_TRACE_UNTIL_FULL) {
    rtl_printf("%s: [BROK] :full stream attribute set and get different\n");
    return -1;
  }
  
  
#ifdef _POSIX_TRACE_TOLOG
  if ((error = posix_trace_attr_getlogfullpolicy(&attr, &full_policy)) != 0) {
    rtl_printf("%s: [BROK] :posix_trace_attr_getstreamfullpolicy returned %d\n", 
	       TEST_NAME, error);
    return -1;
    if (full_policy != POSIX_TRACE_LOOP 
	&& full_policy != POSIX_TRACE_UNTIL_FULL
	&& full_policy != POSIX_TRACE_APPEND
	) {
      rtl_printf("%s: [BROK] :wrong policy returned by posix_trace_attr_getlogfullpolicy\n", 
	       TEST_NAME);
      return -1;
    }
  }


  if ((error = posix_trace_attr_setlogfullpolicy(&attr, POSIX_TRACE_APPEND)) != 0) {
    rtl_printf("%s: [BROK] :posix_trace_attr_setlogfullpolicy returned %d\n", 
	       TEST_NAME, error);
    return -1;
  }  
   
  posix_trace_attr_getlogfullpolicy(&attr, &full_policy);
  if (full_policy != POSIX_TRACE_APPEND) {
    rtl_printf("%s: [BROK] :full log attribute set and get different\n", TEST_NAME);
   return -1;
  }
#endif 


  if ((error = posix_trace_attr_getmaxusereventsize(&attr, sizeof(int), &size)) != 0) {
    rtl_printf("%s: [BROK] :posix_trace_attr_getmaxusereventsize returned %d\n", 
	       TEST_NAME, error);
    return -1;
  }
  if (size <= sizeof(int)) {
      rtl_printf("%s: [BROK] :size in posix_trace_attr_getmaxusereventsize too small\n", TEST_NAME);
      return -1;
  }


  if ((error = posix_trace_attr_getmaxsystemeventsize(&attr, &size)) != 0) {
    rtl_printf("%s: [BROK] :posix_trace_attr_getmaxsystemeventsize returned %d\n", 
	       TEST_NAME, error);
    return -1;
  }

  
  if ((error = posix_trace_attr_getmaxdatasize(&attr, &size)) != 0) {
    rtl_printf("%s: [BROK] :posix_trace_attr_getmaxdatasize returned %d\n", 
	       TEST_NAME, error);
    return -1;
  }

#ifdef _POSIX_TRACE_EVENT_SIZE
  size_set = (size_t) _POSIX_TRACE_EVENT_SIZE;
#else
  size_set = (size_t) 4096; /* Some large arbitrary value (4Kb) for an event */
#endif

  if ((error = posix_trace_attr_setmaxdatasize(&attr, size_set)) != 0) {
    rtl_printf("%s: [BROK] :posix_trace_attr_setmaxdatasize returned %d\n", 
	       TEST_NAME, error);
    return -1;
  }

  posix_trace_attr_getmaxdatasize(&attr, &size);
  if (size < size_set) {
    rtl_printf("%s: [BROK] :event max size too small after setting it\n", TEST_NAME);
    return -1;
  }
    

  if ((error = posix_trace_attr_getstreamsize(&attr, &size)) != 0) {
    rtl_printf("%s: [BROK] :posix_trace_attr_getstreamsize returned %d\n", 
	       TEST_NAME, error);
    return -1;
  }


#ifdef _POSIX_TRACE_STREAM_SIZE
  size_set = (size_t) _POSIX_TRACE_STREAM_SIZE;
#else
  size_set = (size_t) 20971520; /* Some large arbitrary value (20Mb) for a stream*/
#endif

  if ((error = posix_trace_attr_setstreamsize(&attr, size_set)) != 0) {
    rtl_printf("%s: [BROK] :posix_trace_attr_setmaxdatasize returned %d\n", 
	       TEST_NAME, error);
    return -1;
  }

  posix_trace_attr_getstreamsize(&attr, &size);
  if (size < size_set) {
    rtl_printf("%s: [BROK] :stream max size too small after setting it\n", TEST_NAME);
    return -1;
  }
  

  
#ifdef _POSIX_TRACE_TOLOG
  if ((error = posix_trace_attr_getlogsize(&attr, &size)) != 0) {
    rtl_printf("%s: [BROK] :posix_trace_attr_getlogsize returned %d\n", 
	       TEST_NAME, error);
    return -1;
  }

  
  if ((error = posix_trace_attr_setlogsize(&attr, (size_t) 4096)) != -EINVAL) {
    /* This implementation always returns -EINVAL, since it is not supported yet */
    rtl_printf("%s: [BROK] :posix_trace_attr_setlogsize returned %d\n", 
	       TEST_NAME, error);
    return -1;
  }
#endif


  if ((error = posix_trace_attr_destroy(&attr)) != 0) {
    rtl_printf("%s: [BROK] :posix_trace_attr_destroy returned %d\n", 
	       TEST_NAME, error);
    return -1;
  }
  


  /* No errors. All tests were fine. Return 0. */
  rtl_printf("%s: PASS :Stream attribute get/set API works OK\n", TEST_NAME);  
  return 0;

}



void cleanup_module(void) {

}
